﻿/*! local-search.js
 * Searches a Doxygen index file and displays matching pages,
 * on a local filing system without HTTP.
 *
 * @author Mitchell Fraser
 *
 */


/* give the short names used to reduce the size of
  local-search-index.js more meaningful names */
var theIndex = x;

var words = w;

var urls = u;

function computeIndex(word)
{

  // Simple hashing that allows for substring search
  if (word.length <2) return -1;
  // high char of the index
  hi = word.charCodeAt(0);
  if (hi == 0) return -1;
  // low char of the index
  lo = word.charCodeAt(1);
  if (lo==0) return -1;


  return hi*256+lo;
}


function getParam( param )
{
    //function copied from
    //http://www.netlobo.com/url_query_string_javascript.html
    // use permitted by comment in
    // http://www.netlobo.com/comments/url_query_string_javascript

    param = param.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");

    var regexS = "[\\?&]" + param + "=([^&#]*)";
    var regex = new RegExp( regexS );
    var results = regex.exec( window.location.href );
    if( results == null )
        return "";
    else
        return results[1];
}


var g_header;

var g_footer;

var g_top;

var scriptElements;

function stringOf(obj)
{
    var theString = "";
    var attr;
    for (attr in obj)
    {
        theString += attr + " : " +  obj[attr] + "\n";
    }
    return theString

}


function saveHeader()
{
    headerElem = document.getElementsByTagName("head")[0];
    if (null == headerElem)
        g_header = "Error, could not find \"head\" element.";
    else
    {
        //we replace the "local-search-index.js" with ""
        // to avoid loading and parsing the index again
        g_header = (headerElem.innerHTML).replace("\"local-search-index.js\"", "\"\"");

    }

}


function saveTop()
{
    topDiv = document.getElementById("top");
    if (null == topDiv)
       g_top = "Error, no \"top\" div found. <p>\n";
    else
       g_top = topDiv.innerHTML;
}


function saveFooter()
{

    footerDiv = document.getElementById("doxy-footer");

    if (null == footerDiv)
        g_footer = "<hr size=\"1\"><address style=\"align: right;\"><small> Error, no \"footer\" <div> found.</small>";
    else
        g_footer = footerDiv.innerHTML;

}


function writeTop()
{
    document.write("<div class\"navigation\" id=\"top\"");

    document.write(g_top);

    document.write("</div>");
    document.write("<p>");
}

function writeHeader()
{


    document.write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
    document.write("<html><head>\n");

    document.write(g_header);

    document.write("</head>");



}


function writeFooter(){

     document.write("<div id=\"footer\">\n");
     document.write(g_footer);
     document.write("</div>\n");

}


function searchForWord( searchWord, statsList)
{

    hashIndex = computeIndex(searchWord);


    indexEntry  = theIndex[hashIndex];


    var start = statsList.length;
    var count = start;

    var totalHi = 0;
       var totalFreqHi = 0;
       var totalFreqLo = 0;


    if (indexEntry != null)
    {  //found words matching the hash key

        for (var i = 0; i< indexEntry.length; i++)
        {
            foundWord = indexEntry[i];
            if (null == foundWord)
                continue;
            if (  foundWord.indexOf(searchWord) != -1)
            {
                //look up the words array for the document list
                var multiplier = 1;

                var statInfo = new Object();
                statInfo.word = searchWord;
                statInfo.match = foundWord;
                if (foundWord.length == searchWord.length)
                    multiplier = 2;

                statInfo.docs = words[("#" + foundWord)];

                if (null == statInfo.docs)
                   continue;

                for (var j = 0; j < statInfo.docs.length; j++)
                {
                    var matchingDoc = statInfo.docs[j];
                    if (matchingDoc == null)
                    {
                        //document.write("matchingDoc is NULL <p>");
                        continue;
                    }

                   //lsb of freq records whether it is high priority
                    matchingDoc.hi =  matchingDoc.f&1;

                    matchingDoc.f =  matchingDoc.f>>1;
                    var freq = matchingDoc.f;

                    matchingDoc.rank = 0.0;

                    if (matchingDoc.hi) // word occurs in high priority doc
                    {
                       totalHi++;
                       totalFreqHi += freq*multiplier;
                    }
                    else // word occurs in low priority doc
                    {
                        totalFreqLo += freq*multiplier;
                    }

                    matchingDoc.docName = (urls[matchingDoc.x]).n;
                    matchingDoc.docUrl = (urls[matchingDoc.x]).u;

                }

                statsList[count++] = statInfo;

            }
        }

       totalFreq=(totalHi+1)* totalFreqLo + totalFreqHi;
       for (count = start; count < statsList.length; count++)
       {
              statInfo = statsList[count];
              multiplier = 1;
              // whole word matches have a double weight
              if (statInfo.full)
                   multiplier=2;
              for (i=0; i < statInfo.docs.length;i++)
              {
                    docInfo = statInfo.docs;
                    // compute frequency rank of the word in each doc

                    if (docInfo[i] == null)
                        continue;

                    freq = docInfo[i].f;
                    if (docInfo[i].hi > 0)
                    {
                       statInfo.docs[i].rank=
                         (freq*multiplier+totalFreqLo)/totalFreq;
                    }
                    else
                    {
                      statInfo.docs[i].rank=
                           (freq*multiplier)/totalFreq;
                    }
             }//for (i=0..
        }//for (count=...
    } //if (indexEntry != null)

} //searchForWord



function report_results(docs, query)
{


  writeHeader();
  document.write("<body>");
  writeTop();

  document.write("&nbsp;");
  document.write("<div class=\"searchResults\">");

  document.write("<table cellspacing=\"2\">\n");
  document.write("  <tr>\n");
  document.write("    <td colspan=\"2\"><h2>" + search_results() + "</h2></td>\n");
  document.write("  </tr>\n");

  var numDocs = 0;
  for (docsKey in docs)
     numDocs++;

  if (numDocs==0)
  {
    document.write("  <tr>\n");
    document.write("    <td colspan=\"2\">" + matches_text(0, query) + "</td>\n");
    document.write("  </tr>\n");
  }
  else
  {
    document.write("  <tr>\n");
    document.write("    <td colspan=\"2\">" + matches_text(numDocs, query));
    document.write("\n");
    document.write("    </td>\n");
    document.write("  </tr>\n");
    num=1;
    for (var i = 0; i < numDocs; i++)
    {
        doc = docs[i];
        document.write("  <tr>\n");
        document.write("    <td align=\"right\">" + num + "</td>");
        document.write(    "<td><a class=\"el\" href=\"" + doc.docUrl  +"\">" + doc.docName + "</a></td>\n");
        document.write("  <tr>\n");
        document.write("    <td></td><td class=\"tiny\">" + report_matches() + " ");

        numWords = doc.words.length;

        for (var j =0; j< numWords; j++)
        {
          wordInfo = doc.words[j];
          word = wordInfo.word;
          matchRight = wordInfo.match.substr(word.length, wordInfo.match.length);
          document.write("<b>" + word + "</b>" + matchRight +"(" + wordInfo.freq + ") ");
        }
        document.write("    </td>\n");
        document.write("  </tr>\n");
        num++;
    }
  }
  document.write("</table>\n");

  document.write("</div>"); //contents

  writeFooter();

}



function compare_rank(a,b)
{
      if (a.rank == b.rank)
      {
        return 0;
      }
      return (a.rank > b.rank) ? -1 : 1;
}

function sort_results(docs)
{

  return docs.sort(compare_rank);

}





function combine_results(results,docs)
{

    numResults = results.length;

    keyedDocs = new Array();

    for (var i =0; i< numResults; i++)
    {
        wordInfo = results[i];
        docsList = wordInfo.docs;
        numDocs = docsList.length;
        for (var j = 0; j< numDocs; j++)
        {
            di = docsList[j];
            if (di == null)
                continue;
            docKey = di.docUrl;
            rank = di.rank;

            if (keyedDocs[docKey] != null)
            {
                keyedDocs[docKey].rank += rank;

            }else
            {
                docInfo = new Object();
                docInfo.docUrl = docKey;
                docInfo.docName = di.docName;
                docInfo.rank = rank;
                docInfo.words = new Array();

                keyedDocs[docKey] = docInfo;
            }

            wi = new Object();
            wi.word = wordInfo.word;
            wi.match = wordInfo.match;
            wi.freq = di.f;

            idx = keyedDocs[docKey].words.length;
            keyedDocs[docKey].words[idx] = wi;
        }

    }


    var i = 0;
    for (docKey in keyedDocs)
    {
       // document.write("docKey = " + docKey + "<p>");
        docs[i] = keyedDocs[docKey];
        i++;
    }
    return docs;

}




function filter_results(docs, requiredWords, forbiddenWords)
{
    filteredDocs = new Array();
    var numPassed = 0;


    for (var i = 0; i < docs.length; i++)
    {
        copyIt = true;
        words = docs[i].words;


        for (var j = 0; j< requiredWords.length; j++)
        {
            reqWord = requiredWords[j];
            found = false;

            for (var k = 0; k < words.length; k++)
            {
                found = (words[k].word == reqWord);
                if (found)
                    break;
            }
            if (!found)
            {
                copyIt = false; // document fails to contain one of the
                              // required words.
                break;
            }
        }

        if (copyIt)
        {   //don't bother checking for forbidden words
            //if it has failed the required words test

            for (var j = 0; j< forbiddenWords.length; j++)
            {
                forbiddenWord = forbiddenWords[j];
                found = false;
                for (var k = 0; k < words.length; k++)
                {
                    found = (words[k].word == forbiddenWord);
                    if (found)
                    {
                        copyIt = false;
                        break;
                    }
                }
            }
        }

        if (copyIt)
        {
            filteredDocs[numPassed] = docs[i];
            numPassed++;
        }

    }
    return filteredDocs;

}





function doSearch( theIndex, param)
{

    saveHeader();
    saveTop();
    saveFooter();
    indexArray = theIndex;

    searchTerms = getParam(param);



    //indexText =  document.getElementById("index").innerHTML;
    //window.alert(indexText);

    var statsList = new Array();

    requiredWords = new Array();
    forbiddenWords = new Array();
    searchWords   = searchTerms.split("+");

    foundWords = new Array();

    for (var i = 0; i< searchWords.length; i++)
    {
        word = unescape(searchWords[i]);
        word = word.toLowerCase();
        if (word[0] == '+')
        {
            word = word.substr(1);
            requiredWords[requiredWords.length] = word;
          //  document.write("required = " + word + "<p>");
        }
        if (word[0] == '-')
        {
            word = word.substr(1);
            forbiddenWords[forbiddenWords.length] = word;
           // document.write("forbidden = " + word + "<p>");
        }

        if (foundWords[word] == null)
        {
            searchForWord( word, statsList);
            foundWords[word] = i;
        }

    }


    docs = new Array();
    combine_results(statsList, docs);

    filteredDocs = filter_results(docs, requiredWords, forbiddenWords);




    filteredDocs =  sort_results(filteredDocs);

    report_results(filteredDocs, unescape(searchWords.join(" ")));




}



function fillSearchBox(param)
{
    searchTerms = getParam(param);
    searchWords   = searchTerms.split("+");
    searchQuery = searchWords.join(" ");

    queryElement = document.getElementById("query");
    if (null != queryElement)
        queryElement.value = unescape(searchQuery);
    else
        window.alert("queryElement is null");

}




